# -*- coding: utf-8 -*-
import datetime

from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.db import models
from django.test import TestCase
from dojoserializer import DojoDataJSONResponse, serialize


class GenericSomething(models.Model):
    content_type = models.ForeignKey(ContentType,
        related_name='generic_somethings')
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    class Meta:
        app_label = 'dojoserializer'


class TestModel(models.Model):
    """An ordinary django model to test with"""
    booleanfield = models.BooleanField(default=True)
    charfield = models.CharField(max_length=200, default='Foo')
    datefield = models.DateField(default=datetime.date(2009, 7, 3))
    datetimefield = models.DateTimeField(default=datetime.datetime(2009, 7, 3,
        13, 59, 00))
    decimalfield = models.DecimalField(max_digits=5, decimal_places=2,
        default='123.45')
    floatfield = models.FloatField(default=123.45)
    integerfield = models.IntegerField(default=123)
    nullbooleanfield = models.NullBooleanField(default=None)
    timefield = models.TimeField(default=datetime.time(23, 59, 59))
    urlfield = models.URLField(verify_exists=False,
        default='http://www.foo.bar')
    # Relationships
    another_testmodel = models.ForeignKey('self', blank=True, null=True)
    many_other_testmodels = models.ManyToManyField('self', blank=True,
        null=True)
    # Generic relationships
    generic_somethings = generic.GenericRelation(GenericSomething)
    class Meta:
        app_label = 'dojoserializer'
    
    def __unicode__(self):
        return u'%s' % self.charfield
        

class DojoSerializerTestCase(TestCase):
    def setUp(self):
        self.tm_1 = TestModel.objects.create(charfield='tm_1')
        self.tm_2 = TestModel.objects.create(charfield='tm_2')
        self.tm_3 = TestModel.objects.create(charfield='tm_3')
        self.testmodel = TestModel.objects.create(charfield='testmodel',
            another_testmodel=self.tm_1)
        self.testmodel.many_other_testmodels.add(self.tm_2, self.tm_3)
        self.testmodel.generic_somethings.create()
        self.testmodel.generic_somethings.create()
        self.testmodel.generic_somethings.create()
    
    def test_setup(self):
        self.assertTrue(self.tm_1)
        self.assertTrue(self.tm_2)
        self.assertTrue(self.tm_3)
        self.assertTrue(self.testmodel)
        self.assertTrue(self.testmodel.another_testmodel)
        self.assertEquals(self.testmodel.many_other_testmodels.count(), 2)
        self.assertEquals(self.testmodel.generic_somethings.count(), 3)
        
    def test_serialize(self):
        ser_testmodel = serialize(self.testmodel)
        ref_json = """{}&&\n{"numRows": 1, "items": [{"datefield": "2009-07-03", "integerfield": 123, "generic_somethings": [1, 2, 3], "many_other_testmodels": [2, 3], "id": 4, "_unicode": "testmodel", "urlfield": "http://www.foo.bar", "nullbooleanfield": null, "floatfield": 123.45, "timefield": "23:59:59", "charfield": "testmodel", "_pk": 4, "booleanfield": true, "datetimefield": "2009-07-03T13:59:00", "another_testmodel": 1, "decimalfield": "123.45"}], "identifier": "_pk", "label": "_unicode"}"""
        self.assertEquals(ser_testmodel, ref_json)
        
    def test_serialize_id_only(self):
        ser_testmodel = serialize(self.testmodel, fields=('id',))
        ref_json = """{}&&\n{"numRows": 1, "items": [{"_unicode": "testmodel", "id": 4, "_pk": 4}], "identifier": "_pk", "label": "_unicode"}"""
        self.assertEquals(ser_testmodel, ref_json)
        
    def test_serialize_id_and_charfield_but_exclude_charfield(self):
        ser_testmodel = serialize(self.testmodel, fields=('id', 'charfield',),
            exclude_fields=('charfield',))
        ref_json = """{}&&\n{"numRows": 1, "items": [{"_unicode": "testmodel", "id": 4, "_pk": 4}], "identifier": "_pk", "label": "_unicode"}"""
        self.assertEquals(ser_testmodel, ref_json)
        
    def test_serialize_exclude_urlfield_with_model_info(self):
        ser_testmodel = serialize(self.testmodel, exclude_fields=('urlfield',),
            add_model_info=True)
        ref_json = """{}&&\n{"numRows": 1, "items": [{"datefield": "2009-07-03", "integerfield": 123, "generic_somethings": [1, 2, 3], "many_other_testmodels": [2, 3], "id": 4, "_unicode": "testmodel", "nullbooleanfield": null, "floatfield": 123.45, "timefield": "23:59:59", "charfield": "testmodel", "_pk": 4, "booleanfield": true, "datetimefield": "2009-07-03T13:59:00", "another_testmodel": 1, "decimalfield": "123.45"}], "identifier": "_pk", "label": "_unicode"}"""
        self.assertEquals(ser_testmodel, ref_json)
        
    def test_dojodatajsonresponse(self):
        response = DojoDataJSONResponse(self.testmodel)
        self.assertEquals(response.content, serialize(self.testmodel))
        self.assertEquals(response.status_code, 200)
        self.assertTrue(response.has_header('Last-Modified'))
        self.assertTrue(response.has_header('Pragma'))
        self.assertTrue(response.has_header('Cache-Control'))
        self.assertTrue(response.has_header('Content-Type'))
        self.assertTrue(response.has_header('Expires'))
        
    def test_test_dojodatajsonresponse_argumentpassing(self):
        response = DojoDataJSONResponse(self.testmodel, ('id', 'charfield',),
            ('charfield',), True)
        ser_testmodel = serialize(self.testmodel, ('id', 'charfield',),
            ('charfield',), True)
        self.assertEquals(response.content, ser_testmodel)


# -*- coding: utf-8 -*-
import datetime
from django.db.models import Model
from django.db.models.query import QuerySet
from django.http import HttpResponse
from django.utils import simplejson

def _json_safe_model_dict(model_instance, fields=(), exclude_fields=(),
    add_model_info=False):
    """
    Returns a dict of JSON safe formatted model fields. Related model objects
    are represented by their primary keys.
    
    Arguments:
        fields = A tuple of field names to use. An empty tuple includes all.
        exclude_fields = A tuple of field names that are excluded.
        add_model_info = A boolean indicating if _app_label and _model_name
            should be added to dictionary
    
    Two additional key-value-pairs are set to have a generic identifier and
    label for each item:
        _pk: The primary key of a model
        _unicode: The string representation of a model
    """
    data = {
        '_pk': model_instance.pk,
        '_unicode': unicode(model_instance),
    }
    if add_model_info:
        data.update({
            '_app_label': model_instance._meta.app_label,
            '_model_name': model_instance._meta.module_name,
        })
    # Standard fields
    for field in model_instance._meta.fields:
        if fields and field.name not in fields:
            continue
        if exclude_fields and field.name in exclude_fields:
            continue
        attr = getattr(model_instance, field.name)
        if isinstance(attr, datetime.datetime):
            attr = str(attr).replace(' ', 'T')
        elif isinstance(attr, datetime.date):
            attr = str(attr)
        elif isinstance(attr, Model):
            attr = attr.id
        elif attr is None:
            attr = None
        elif isinstance(attr, (str, unicode, int, long, float, bool,)):
            attr = attr
        else:
            attr = str(attr)
        data.update({ field.name: attr })
    # Many to many fields
    for field in model_instance._meta.many_to_many:
        if fields and field.name not in fields:
            continue
        if exclude_fields and field.name in exclude_fields:
            continue
        attr = getattr(model_instance, field.name)
        data.update({ field.name: list(attr.values_list('pk', flat=True)) })
    return data

def serialize(model_instances, fields=(), exclude_fields=(),
    add_model_info=False):
    """
    Serializes a model instance, a collection of model instances or a queryset
    to a dojo data compatible JSON string. To prevent JSON hijacking, the
    string gets prefixed, see http://tinyurl.com/sitepensiajax for details.
    
    Two additional key-value-pairs are set to have a generic identifier and
    label for each item:
        _pk: The primary key of a model
        _unicode: The string representation of a model
    
    Arguments:
        model_instances = One or more model instances or a queryset
        fields = A tuple of field names to use. An empty tuple includes all.
        exclude_fields = A tuple of field names that are excluded.
        add_model_info = A boolean indicating if _app_label and _model_name
            should be added to dictionary
    """
    if isinstance(model_instances, QuerySet):
        model_instances = list(model_instances)
    if not isinstance(model_instances, list):
        model_instances = [ model_instances ]
    items = [_json_safe_model_dict(i, fields, exclude_fields) for i in \
        model_instances if isinstance(i, Model)]
    dojo_data_dict = {
        'identifier': '_pk',
        'label': '_unicode',
        'numRows': len(items),
        'items': items,
    }
    dojo_data_json = simplejson.dumps(dojo_data_dict, ensure_ascii=False)
    return ''.join(['{}&&\n', dojo_data_json])
    
def DojoDataJSONResponse(model_instances, fields=(), exclude_fields=(),
    add_model_info=False, status=200):
    """
    Serializes a model instance, a collection of model instances or a queryset
    to a dojo data compatible HTTP Response. To prevent JSON hijacking, it
    gets prefixed, see http://tinyurl.com/sitepensiajax for details.
    The HTTP Headers are set to ensure the response does not get cached.
    
    Two additional key-value-pairs are set to have a generic identifier and
    label for each item:
        _pk: The primary key of a model
        _unicode: The string representation of a model
    
    Arguments:
        model_instances = One or more model instances or a queryset
        fields = A tuple of field names to use. An empty tuple includes all.
        exclude_fields = A tuple of field names that are excluded.
        add_model_info = A boolean indicating if _app_label and _model_name
            should be added to dictionary
    """
    dojo_data_json = serialize(model_instances, fields, exclude_fields,
        add_model_info)
    response = HttpResponse(content=dojo_data_json, 
        mimetype='application/json', status=status)
    response['Cache-Control'] = 'no-cache, no-store, max-age=0, must-revalidate'
    response['Pragma'] = 'no-cache'
    response['Expires'] = 'Mon, 01 Jan 1990 00:00:00 GMT'
    response['Last-Modified'] = datetime.datetime.strftime(
        datetime.datetime.utcnow(), "%a, %d %b %Y %H:%M:%S GMT")
    return response


# -*- coding: utf-8 -*-
from django.contrib.contenttypes.models import ContentType
from django.core.paginator import Paginator
from django.db import models
from django.db.models.fields.related import ForeignKey
from django.http import HttpResponse, HttpResponseBadRequest, \
    HttpResponseForbidden, HttpResponseNotAllowed, HttpResponseNotFound
from django.shortcuts import get_object_or_404
from django.views.generic.create_update import get_model_and_form_class
from dojoserializer import DojoDataJSONResponse

def create_object(request, model=None, form_class=None, login_required=False,
    fields=(), exclude_fields=(), add_model_info=False):
    """
    Creates a valid model instance and returns it dojo data serialized. 
    """
    if login_required and not request.user.is_authenticated():
        return HttpResponseForbidden()
    if request.method in ['POST', 'PUT']:
        model, form_class = get_model_and_form_class(model, form_class)
        data = model().__dict__              # default values
        data.update(request.POST.items())
        data.update(request.FILES.items())
        form = form_class(data)
        if form.is_valid():
            new_obj = form.save()
            return DojoDataJSONResponse(new_obj, fields=fields, exclude_fields=\
                exclude_fields, add_model_info=add_model_info, status=201)
        else:
            return HttpResponseBadRequest(content=form.errors)
    else:
        return HttpResponseNotAllowed()

def read_object(request, model, object_id, login_required=False, fields=(),
    exclude_fields=(), add_model_info=False):
    """
    Reads and returns a model instance and returns it dojo data serialized.
    """
    if login_required and not request.user.is_authenticated():
        return HttpResponseForbidden()
    if request.method in ['GET', 'HEAD']:
        obj = get_object_or_404(model, pk=object_id)
        if request.method == 'GET':
            return DojoDataJSONResponse(obj, fields=fields, exclude_fields=\
                exclude_fields, add_model_info=add_model_info)
        return HttpResponse()
    else:
        return HttpResponseNotAllowed()
        
def update_object(request, object_id, model=None, form_class=None,
    login_required=False, fields=(), exclude_fields=(), add_model_info=False):
    """
    Updates and returns a model instance dojo data serialized.
    """
    if login_required and not request.user.is_authenticated():
        return HttpResponseForbidden()
    if request.method in ['POST', 'PUT']:
        model, form_class = get_model_and_form_class(model, form_class)
        obj = get_object_or_404(model, pk=object_id)
        data = { }
        fk_fields = [fk_field for fk_field in obj._meta.fields if isinstance(
            fk_field, ForeignKey)]
        for fk_field in fk_fields:
            foreign_obj = getattr(obj, fk_field.name)
            if foreign_obj and foreign_obj.id:
                data.update({ fk_field.name: foreign_obj.id })
        data.update(dict(request.REQUEST.items()))
        data.update(request.FILES.items())
        obj.__dict__.update(data)
        form = form_class(obj.__dict__)
        if form.is_valid():
            obj.save()
            return DojoDataJSONResponse(obj, fields=fields, exclude_fields=\
                exclude_fields, add_model_info=add_model_info)
        else:
            return HttpResponseBadRequest()
    else:
        return HttpResponseNotAllowed()
        
def delete_object(request, model, object_id, login_required=False):
    """
    Deletes a model instance.
    """
    if login_required and not request.user.is_authenticated():
        return HttpResponseForbidden()
    if request.method == 'DELETE':
        obj = get_object_or_404(model, pk=object_id)
        obj.delete()
        return HttpResponse(status=204)
    else:
        return HttpResponseNotAllowed()
        
def object_list(request, queryset, paginate_by=None, page=None,
    login_required=False, allow_empty=False, fields=(), exclude_fields=(),
    add_model_info=False):
    """
    Returns a dojo serialized list of model instances defined in a queryset.
    
    Allow_empty returns the status code 200 if lists are empty, when set to
    False this would return status code 404 (which is the default behavior).
    
    Using the arguments or request parameters 'paginate_by' (default: 10) and
    'page' (default: 1) you're able to paginate through object lists.
    
    To allow listings to logged in users only, set 'login_required' to True.
    
    To customize the fields of the resulting JSON output, you can populate the
    'fields' and/or 'exclude_fields' tuple with fieldnames. Set 'add_model_info'
    to True to get the objects app_label and module_names.
    """
    if login_required and not request.user.is_authenticated():
        return HttpResponseForbidden()
    if request.method in ['GET', 'HEAD']:
        if not page:
            try:
                page = int(request.GET.get('page', '1'))
            except ValueError:
                page = 1
        if not paginate_by:
            try:
                paginate_by = int(request.GET.get('paginate_by', '10'))
            except ValueError:
                paginate_by = 10
        try:
            paginator = Paginator(queryset, paginate_by)
            objects = paginator.page(page).object_list
        except:
            return HttpResponseBadRequest()
        if not allow_empty and not objects:
            return HttpResponseNotFound()
        elif request.method == 'GET':
            return DojoDataJSONResponse(objects, fields=fields,
                exclude_fields=exclude_fields, add_model_info=add_model_info)
        elif request.method == 'HEAD':
            return HttpResponse()
    else:
        return HttpResponseNotAllowed()
        
def object_list_filtered_by_generic_fk(request, related_object_id,
    related_object_model=None, related_object_app_label=None,
    related_object_model_name=None, related_object_id_field_name='object_id',
    related_object_content_type_field_name='content_type',
    query_object_model=None, query_object_app_label=None,
    query_object_model_name=None, allow_empty=False, paginate_by=None,
    page=None, login_required=False, fields=(), exclude_fields=(),
    add_model_info=False):
    """
    Returns a dojo serialized list of model instances filtered by a generic
    relation.
    
    E.g. you can return all BlogPost (the query object) instances that are
    tagged with the Tag (the related object) 'Foo', which has the id 4 (the
    related_object_id).
    
    Both the model for the query object and the related object can be passed
    directly ('query_object_model' and 'related_object_model') or indirectly
    using their app label and model name values ('query_object_app_label' and
    'query_object_model_name' for the query object and
    'related_object_app_label' and 'related_object_model_name' for the related
    object).
    
    Allow_empty returns the status code 200 if lists are empty, when set to
    False this would return status code 404 (which is the default behavior).
    
    Using the arguments or request parameters 'paginate_by' (default: 10) and
    'page' (default: 1) you're able to paginate through object lists.
    
    To allow listings to logged in users only, set 'login_required' to True.
    
    To customize the fields of the resulting JSON output, you can populate the
    'fields' and/or 'exclude_fields' tuple with fieldnames. Set 'add_model_info'
    to True to get the objects app_label and module_names.
    """
    if request.method in ['GET', 'HEAD']:
        if not query_object_model and query_object_app_label and \
            query_object_model_name:
            query_object_model = models.get_model(query_object_app_label,
                query_object_model_name)
        if not related_object_model and related_object_app_label and \
            related_object_model_name:
            related_object_model = models.get_model(related_object_app_label,
                related_object_model_name)
        if not query_object_model or not related_object_model:
            return HttpResponseBadRequest()
        related_object_content_type = ContentType.objects.get_for_model(
            related_object_model)
        try:
            filters = {
                related_object_content_type_field_name: \
                    related_object_content_type.id,
                related_object_id_field_name: related_object_id,
            }
            queryset = query_object_model._default_manager.filter(**filters)
            if not allow_empty and not queryset:
                return HttpResponseNotFound()
            if request.method == 'HEAD':
                return HttpResponse()
            return object_list(request, queryset, paginate_by=paginate_by,
                page=page, login_required=login_required,
                allow_empty=allow_empty, fields=fields,
                exclude_fields=exclude_fields, add_model_info=add_model_info)
        except:
            return HttpResponseBadRequest()
    else:
        return HttpResponseNotAllowed()
        
def object_list_filtered_by_fk(request, related_object_id,
    related_object_field_name=None, query_object_model=None,
    query_object_app_label=None, query_object_model_name=None,
    allow_empty=False, paginate_by=None, page=None, login_required=False,
    fields=(), exclude_fields=(), add_model_info=False):
    """
    Returns a dojo serialized list of model instances filtered by a foreign
    key relation.
    
    E.g. you can return all Comment (the query object) instances that are
    related to a BlogPost (the related object) 'First Post', which has the id
    3 (the related_object_id).
    
    The model of the query object can be passed directly ('query_object_model')
    or indirectly using its app label and model name values (
    'query_object_app_label' and 'query_object_model_name').
    
    Allow_empty returns the status code 200 if lists are empty, when set to
    False this would return status code 404 (which is the default behavior).
    
    Using the arguments or request parameters 'paginate_by' (default: 10) and
    'page' (default: 1) you're able to paginate through object lists.
    
    To allow listings to logged in users only, set 'login_required' to True.
    
    To customize the fields of the resulting JSON output, you can populate the
    'fields' and/or 'exclude_fields' tuple with fieldnames. Set 'add_model_info'
    to True to get the objects app_label and module_names.
    """
    if request.method in ['GET', 'HEAD']:
        if not query_object_model and query_object_app_label and \
            query_object_model_name:
            query_object_model = models.get_model(query_object_app_label,
                query_object_model_name)
        if not query_object_model or not related_object_id or not \
            related_object_field_name:
            return HttpResponseBadRequest()
        try:
            filters = {
                related_object_field_name: related_object_id,
            }
            queryset = query_object_model._default_manager.filter(**filters)
            if not allow_empty and not queryset:
                return HttpResponseNotFound()
            if request.method == 'HEAD':
                return HttpResponse()
            return object_list(request, queryset, paginate_by=paginate_by,
                page=page, login_required=login_required,
                allow_empty=allow_empty, fields=fields,
                exclude_fields=exclude_fields, add_model_info=add_model_info)
        except Exception:
            return HttpResponseBadRequest()
    else:
        return HttpResponseNotAllowed()
        
def restful_dispatcher(request, model, form_class=None, object_id=None,
    login_required=False, fields=(), exclude_fields=(), add_model_info=False):
    """
    Dispatches requests in a RESTful manner.
    """
    if login_required and not request.user.is_authenticated():
        return HttpResponseForbidden()
    if request.method in ['POST', 'PUT'] and not object_id:
        return create_object(request, model=model, form_class=form_class,
            login_required=login_required, fields=fields, exclude_fields=\
            exclude_fields, add_model_info=add_model_info)
    elif request.method in ['POST', 'PUT'] and object_id:
        return update_object(request, model=model, form_class=form_class,
            object_id=object_id, login_required=login_required, fields=fields,
            exclude_fields=exclude_fields, add_model_info=add_model_info)
    elif request.method == 'DELETE':
        if not object_id:
            return HttpResponseBadRequest()
        return delete_object(request, model=model, object_id=object_id,
            login_required=login_required)
    elif request.method in ['GET', 'HEAD']:
        if object_id:
            return read_object(request, model=model, object_id=object_id,
                login_required=login_required, fields=fields,
                exclude_fields=exclude_fields, add_model_info=add_model_info)
        else:
            return object_list(request, model._default_manager.all(),
                login_required=login_required, fields=fields,
                exclude_fields=exclude_fields, add_model_info=add_model_info)


from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.template import Library, Node

register = Library()

@register.tag
def content_type_id(parser, token):
    """
    Returns the content type id for the model given by its app_label and its
    model_name.
    """
    app_label, model_name = token.split_contents()[1:]
    return ContentTypeIdNode(app_label, model_name)


class ContentTypeIdNode(Node):
    def __init__(self, app_label, model_name):
        self.app_label = app_label
        self.model_name = model_name
    
    def render(self, context):
        model = models.get_model(self.app_label, self.model_name)
        if not model:
            raise Exception('Check app_label and model_name in template tag!')
        return ContentType.objects.get_for_model(model).id




from setuptools import setup, find_packages

setup(
    name = "django-dojoserializer",
    version = "1.1.2",
    url = 'http://code.google.com/p/django-dojoserializer',
    license = 'MIT',
    description = "Serializes django model instances to a dojo data " + \
        "compatible JSON representation",
    author = 'Fabian Topfstedt',
    author_email = 'myfirstname@mylastname.de',
    package_data = {
        'src/dojoserializer': [
            'templatetags/*.py',
        ]
    },
    packages = find_packages('src'),
    package_dir = {'': 'src'},
    install_requires = ['setuptools'],
)
